#!/usr/bin/perl

$imagefile = 'atari.dsk';

open IN, $imagefile; binmode IN; $secinbuf = -1;

# ICD partition table
for ($i = 0; $i <= 11; $i++) {
   $ipart[$i] = iread(0, 342 + $i*12, 12);
}

# Standard Atari partition table (allows only four!)
for ($i = 0; $i <= 3; $i++) {
   $apart[$i] = iread(0, 454 + $i*12, 12);
}

@part = (@apart, @ipart);

if (-f "$ENV{HOME}/.mtoolsrc") {
   die "I don't want to overwrite your .mtoolsrc, sorry! move it temporarily maybe?"; 
}

open OUT1, ">$ENV{HOME}/.mtoolsrc";
open OUT2, ">source-me";
if (! -d 'atari') {mkdir ("atari", 0666); }

$pwd = `pwd`; chomp($pwd); $drive = 'c';

print OUT1 "MTOOLS_SKIP_CHECK=1\n";

foreach(@part) {
   /^.(...)/; $code = $1;
   ($c1, $c2, $c3, $c4, $pstart, $plength) =
      unpack("CCCCNN", $_);
   print "partinfo ", hx($_), " "; 
# sanity checks.
   unless (($c1 & 0x01) && ($code =~ /[A-Za-z0-9]{3}/)) {print "skipped\n"; next ;}
   $offs = $pstart * 512;
   print "$code start $pstart ($offs) length $plength\n";
   print OUT1 "drive $drive:\nfile=\"$pwd/$imagefile\"\noffset=$offs\n\n";
   print OUT2 "mkdir atari/$drive\necho copying $drive:\nmcopy -snm \"$drive:/*\" atari/$drive\n";
   $drive++;
}

print "
Wrote atari partition info to $ENV{HOME}/.mtoolsrc.
Type 'cat $ENV{HOME}/.mtoolsrc' if curious!
Wrote copying script to file 'source-me'. It will extract each partition
to a subdirectory under 'atari/' in the current dir. 
Type 'source source-me' to run it!
";
close OUT2; close OUT1;

#
# iread(sector, bytestart, len) read string of bytes (0-511) from sector 
#
# global $secbuf, $secinbuf
#
sub iread {
   local $secnum = $_[0], $start = $_[1], $len = $_[2];
   unless ($secinbuf == $secnum) {
      seek IN, $secnum*512, SEEK_SET;
      read IN, $secbuf, 512;
      $secinbuf = $secnum;
   }
   return substr($secbuf, $start, $len);
}


#
# convert string to hex
#
sub hx {
   local $x = "", $s = $_[0], $i; 
   for ($i = 0; $i < length($s); $i++) {
      $x .= sprintf("%02X", ord(substr($s, $i, 1)) );
   }
   $x;
}

